@@ -1,8 +1,13 @@
 package Media::Convert::Asset;
 
+use strict;
+use warnings;
+use v5.24;
+
 our $VERSION;
 
 use Media::Convert;
+use Carp;
 
 =head1 NAME
 
@@ -119,7 +124,7 @@ sub _probe_mtime {
 	}
 	my @statdata = stat($self->url);
 	if(scalar(@statdata) == 0) {
-		return undef;
+		return;
 	}
 	return $statdata[9];
 }
@@ -141,7 +146,27 @@ sub _probe_duration {
 	if($self->has_reference) {
 		return $self->reference->duration;
 	}
-	return $self->_get_probedata->{format}{duration};
+	if($self->duration_style eq "seconds") {
+		return $self->_get_probedata->{format}{duration};
+	} else {
+		return $self->duration_frames;
+	}
+}
+
+=head2 duration_frames
+
+The number of frames in this asset.
+
+=cut
+
+has 'duration_frames' => (
+	is => 'rw',
+	builder => '_probe_duration_frames',
+	lazy => 1,
+);
+
+sub _probe_duration_frames {
+	return shift->_get_videodata->{duration_ts};
 }
 
 =head2 duration_style
@@ -149,11 +174,19 @@ sub _probe_duration {
 The time unit is used for the C<duration> attribute. One of 'seconds'
 (default) or 'frames'. Will not be probed.
 
+Deprecated; will be removed in a future release. Use duration_frames
+instead of setting this to a non-default value.
+
 =cut
 
+sub _warn_duration {
+	carp "setting duration_style is deprecated. Rather use duration_frames."
+}
+
 has 'duration_style' => (
 	is => 'rw',
 	default => 'seconds',
+	trigger => \&_warn_duration,
 );
 
 =head2 video_codec
@@ -237,7 +270,7 @@ sub _probe_videosize {
 	}
 	my $width = $self->video_width;
 	my $height = $self->video_height;
-	return undef unless defined($width) && defined($height);
+	return unless defined($width) && defined($height);
 	return $self->video_width . "x" . $self->video_height;
 }
 
@@ -344,7 +377,7 @@ sub _probe_videominrate {
 	if(defined($rate)) {
 		return $rate * 0.5;
 	}
-	return undef;
+	return;
 }
 
 =head2 video_maxrate
@@ -379,7 +412,7 @@ sub _probe_video_preset {
 	if($self->has_reference) {
 		return $self->reference->video_preset;
 	}
-	return undef;
+	return;
 }
 
 sub _probe_videomaxrate {
@@ -395,7 +428,7 @@ sub _probe_videomaxrate {
 	if(defined($rate)) {
 		return $rate * 1.45;
 	}
-	return undef;
+	return;
 }
 
 =head2 aspect_ratio
@@ -514,7 +547,7 @@ sub _probe_quality {
 	if($self->has_reference) {
 		return $self->reference->quality;
 	}
-	return undef;
+	return;
 }
 
 =head2 metadata
@@ -620,8 +653,8 @@ sub _probe_blackspots {
 	my $blacks = [];
 	pipe R, W;
 	if(fork == 0) {
-		open STDERR, ">&W";
-		open STDOUT, ">&W";
+		open STDERR, ">", "&W";
+		open STDOUT, ">", "&W";
 		my @cmd = ("ffmpeg", "-threads", "1", "-nostats", "-i", $self->url, "-vf", "blackdetect=d=0:pix_th=.01", "-f", "null", "/dev/null");
 		exec @cmd;
 		die "exec failed";
@@ -668,6 +701,27 @@ sub _probe_astream_ids {
 	return \@rv;
 }
 
+=head2 audio_channel_count
+
+Returns the number of channels in the first audio stream
+
+=cut
+
+has 'audio_channel_count' => (
+	is => 'rw',
+	isa => 'Int',
+	lazy => 1,
+	builder => '_probe_audio_channel_count',
+);
+
+sub _probe_audio_channel_count {
+	my $self = shift;
+	if($self->has_reference) {
+		return $self->reference->audio_channel_count;
+	}
+	return $self->_get_audiodata->{channels};
+}
+
 =head2 vstream_id
 
 Returns the numeric ID for the first video stream in this file. Useful
@@ -688,9 +742,23 @@ sub _probe_vstream_id {
 
 =head2 extra_params
 
-Add extra parameters. This should be used sparingly, rather add some
+Add extra (output) parameters. This should be used sparingly, rather add some
 abstraction.
 
+=head3 handles
+
+=over
+
+=item *
+
+add_param (adds an extra parameter)
+
+=item *
+
+drop_param (delete a parameter)
+
+=back
+
 =cut
 
 has 'extra_params' => (
@@ -713,6 +781,47 @@ sub _probe_extra_params {
 	return {};
 }
 
+=head2 input_params
+
+Add extra input parameters. This should be used sparingly, rather add
+some abstraction.
+
+=head3 handles
+
+=over
+
+=item *
+
+add_input_param (adds an extra parameter)
+
+=item *
+
+drop_input_param (delete a parameter)
+
+=back
+
+=cut
+
+has 'input_params' => (
+	traits => ['Hash'],
+	isa => 'HashRef[Str]',
+	is => 'ro',
+	handles => {
+		add_input_param => 'set',
+		drop_input_param => 'delete',
+	},
+	builder => "_probe_input_params",
+	lazy => 1,
+);
+
+sub _probe_input_params {
+	my $self = shift;
+	if($self->has_reference) {
+		return $self->reference->input_params;
+	}
+	return {};
+}
+
 =head2 time_offset
 
 Apply an input time offset to this video (only valid when used as an
@@ -762,6 +871,13 @@ sub readopts {
 	if($self->has_time_offset) {
 		push @opts, ("-itsoffset", $self->time_offset);
 	}
+
+	if(scalar(keys(%{$self->input_params})) > 0) {
+		foreach my $param(keys %{$self->input_params}) {
+			push @opts, ("-$param", $self->input_params->{$param});
+		}
+	}
+
 	push @opts, ("-i", $self->url);
 	return @opts;
 }
@@ -815,6 +931,8 @@ sub writeopts {
 		} else {
 			push @opts, ('-frames:v', $self->duration);
 		}
+	} elsif(defined($self->duration_frames)) {
+		push @opts, ('-frames:v', $self->duration_frames);
 	}
 	if(defined($self->pix_fmt)) {
 		push @opts, ('-pix_fmt', $self->pix_fmt);
@@ -850,12 +968,12 @@ sub _probe {
 	if($self->has_reference) {
 		return $self->reference->_get_probedata;
 	}
-	open JSON, "-|:encoding(UTF-8)", "ffprobe", "-loglevel", "quiet", "-print_format", "json", "-show_format", "-show_streams", $self->url;
+	open my $jsonpipe, "-|:encoding(UTF-8)", "ffprobe", "-loglevel", "quiet", "-print_format", "json", "-show_format", "-show_streams", $self->url;
 	my $json = "";
-	while(<JSON>) {
+	while(<$jsonpipe>) {
 		$json .= $_;
 	}
-	close JSON;
+	close $jsonpipe;
 	return decode_json($json);
 }
 
